@@ -7,6 +7,7 @@ from datetime import datetime, timedelta
 from os.path import exists, join
 from os import makedirs
 from appdirs import site_data_dir
+from redzoo.database.simple import SimpleDB
 from homeconnect_webthing.auth import Auth
 from homeconnect_webthing.eventstream import EventListener
 from homeconnect_webthing.utils import print_duration, is_success
@@ -41,6 +42,8 @@ class Appliance(EventListener):
         self._power = ""
         self._door = ""
         self._operation = ""
+        self.completed = True
+        self.status = "INACTIVE"
         self.__program_active = ""
         self.child_lock = False
         self._reload_status_and_settings()
@@ -88,7 +91,45 @@ class Appliance(EventListener):
         self.__value_changed_listeners.add(value_changed_listener)
         self._notify_listeners()
 
+    def __update_state(self):
+        if self.power.lower() == "on" and self.operation.lower() == 'delayedstart':
+            new_status = "DELAYED_STARTED"
+
+        elif self.power.lower() == "on" and self.operation.lower() == 'run':
+            new_status = "RUNNING"
+            self.completed = False
+
+        elif self.power.lower() == "on" and \
+             not self.completed and \
+             self.door.lower() == "closed" and \
+             self.remote_start_allowed and \
+             self.operation.lower() not in ['delayedstart', 'run', 'finished', 'inactive']:
+            new_status = "REMOTE_STARTABLE"
+
+        elif self.power.lower() != "on":
+            new_status = "INACTIVE"
+            self.completed = True
+
+        else:
+            new_status = "INACTIVE"
+
+        if self.status != new_status:
+            logging.info("new status: " + new_status + " (previous: " + self.status + ")")
+            self.status = new_status
+
+
     def _notify_listeners(self):
+        self.__update_state()
+
+        # will be removed
+        self.startable = self.door.lower() == "closed" and \
+                         self.power.lower() == "on" and \
+                         self.remote_start_allowed and \
+                         self.operation.lower() not in ['delayedstart', 'run', 'finished', 'inactive']
+        self.started = self.power.lower() == "on" and \
+                       self.door.lower() == "closed" and \
+                       self.operation.lower() in ['delayedstart', 'run']
+
         for value_changed_listener in self.__value_changed_listeners:
             value_changed_listener()
 
@@ -122,18 +163,18 @@ class Appliance(EventListener):
     def _reload_status_and_settings(self):
         self.last_refresh = datetime.now()
         try:
+            status = self._perform_get('/status').get('data', {}).get('status', {})
+            self._on_values_changed(status, "reload status", notify_listeners=False)
+
             settings = self._perform_get('/settings').get('data', {}).get('settings', {})
             self._on_values_changed(settings, "reload settings")
-
-            status = self._perform_get('/status').get('data', {}).get('status', {})
-            self._on_values_changed(status, "reload status")
         except Exception as e:
             if isinstance(e, OfflineException):
                 logging.info(self.name + " is offline. Could not query current status/settings")
             else:
                 logging.warning(self.name + " error occurred on refreshing" + str(e))
 
-    def _on_values_changed(self, changes: List[Dict[str, Any]], source: str):
+    def _on_values_changed(self, changes: List[Dict[str, Any]], source: str, notify_listeners: bool = True):
         if len(changes) > 0:
             for change in changes:
                 key = str(change.get('key', ""))
@@ -143,7 +184,8 @@ class Appliance(EventListener):
                         logging.warning(self.name + " unhandled change " + str(change) + " (" + source + ")")
                 except Exception as e:
                     logging.warning("error occurred by handling change with key " + key + " (" + source + ")" + " " + str(e) + "(" + source + ")")
-        self._notify_listeners()
+        if notify_listeners:
+            self._notify_listeners()
 
     def _on_value_changed(self, key: str, change: Dict[str, Any], source: str) -> bool:
         if key == 'BSH.Common.Status.DoorState':
@@ -298,16 +340,6 @@ class Dishwasher(Appliance):
             return super()._on_value_changed(key, change, source)
         return True
 
-    def _notify_listeners(self):
-        self.startable = self.door.lower() == "closed" and \
-                         self.power.lower() == "on" and \
-                         self.remote_start_allowed and \
-                         self.operation.lower() not in ['delayedstart', 'run', 'finished', 'inactive']
-        self.started = self.power.lower() == "on" and \
-                       self.door.lower() == "closed" and \
-                       self.operation.lower() in ['delayedstart', 'run']
-        super()._notify_listeners()
-
     def read_start_date(self) -> str:
         start_date = datetime.now() + timedelta(seconds=self.__program_start_in_relative_sec)
         if start_date > datetime.now():
@@ -378,29 +410,16 @@ class FinishInAppliance(Appliance):
         self._program_finish_in_relative_sec = 0
         self.__program_finish_in_relative_max_sec = 86000
         self.__program_finish_in_relative_stepsize_sec = 60
-        self.__start_date = ""
+        self._durations = SimpleDB(haid + '_durations', sync_period_sec=0)
         super().__init__(device_uri, auth, name, device_type, haid, brand, vib, enumber)
-        self.__durations = self.__load_durations()
-
-    def __load_durations(self):
-        durations = {}
-        try:
-            with open(self.__file(), 'r') as file:
-                durations = json.load(file)
-                logging.info("config " + str(durations) + " loaded (" + self.__file() + ")")
-        except Exception as e:
-            logging.info("error reading " + self.__file() + " " + str(e))
-        return durations
-
-    def __file(self) -> str:
-        dir = site_data_dir("HomeConnect", appauthor=False)
-        if not exists(dir):
-            makedirs(dir)
-        file = join(dir, self.haid + '_config.json')
-        return file
 
     def _on_value_changed(self, key: str, change: Dict[str, Any], source: str) -> bool:
-        if key == 'BSH.Common.Option.FinishInRelative':  # supported by dryer & washer only
+        if key == 'BSH.Common.Status.OperationState':
+            operation = change.get('value', "undefined")
+            if operation != self._operation:
+                logging.info(self.name + " field 'operation state': " + str(operation) + ". previous state = " + str(self._operation) + " (" + source + ")")
+                self._operation = operation
+        elif key == 'BSH.Common.Option.FinishInRelative':  # supported by dryer & washer only
             if 'value' in change.keys():
                 self._program_finish_in_relative_sec = int(change['value'])
                 logging.info(self.name + " field 'finish in relative': " + str(self._program_finish_in_relative_sec) + " (" + source + ")")
@@ -422,48 +441,36 @@ class FinishInAppliance(Appliance):
             return super()._on_value_changed(key, change, source)
         return True
 
-    def _notify_listeners(self):
-        self.startable = self.door.lower() == "closed" and \
-                         self.power.lower() == "on" and \
-                         self.remote_start_allowed and \
-                         self.program_remote_control_active and \
-                         self.operation.lower() not in ['delayedstart', 'run', 'finished', 'inactive']
-        self.started = self.power.lower() == "on" and \
-                       self.door.lower() == "closed" and \
-                       self.operation.lower() in ['delayedstart', 'run']
-        super()._notify_listeners()
-
     def read_start_date(self) -> str:
-        if self.operation.lower() == 'delayedstart':
-            return self.__start_date
+        if self.operation.lower() == 'delayedstart' and self._program_finish_in_relative_sec > 0:
+            start_date = datetime.now() + timedelta(seconds=self._program_finish_in_relative_sec) - timedelta(seconds=self.__program_duration_sec())
+            return start_date.strftime("%Y-%m-%d %H:%M")
         else:
             return ""
 
     def _program_fingerprint(self) -> str:
         return self._program_selected
 
+    @property
+    def program_duration_hours(self) -> float:
+        return round(self.__program_duration_sec() / (60*60), 1)
+
     def __program_duration_sec(self):
         # will update props, if duration is available
         program_fingerprint = self._program_fingerprint()
         if len(self.program_selected) > 0 and self._program_finish_in_relative_sec > 0 and self.operation.lower() not in ['delayedstart', 'run', 'finished', 'inactive']:
-            if self.__durations.get(program_fingerprint, "?") != self._program_finish_in_relative_sec:   # duration changed?
-                self.__durations[program_fingerprint] = self._program_finish_in_relative_sec
+            if self._durations.get(program_fingerprint, -1) != self._program_finish_in_relative_sec:   # duration changed?
+                self._durations.put(program_fingerprint, self._program_finish_in_relative_sec)
                 logging.info("duration update for " + program_fingerprint + " with " + str(self._program_finish_in_relative_sec))
-                with open(self.__file(), 'w') as file:
-                    json.dump(self.__durations, file)
 
         # get duration
-        duration_sec = self.__durations.get(program_fingerprint, None)
+        duration_sec = self._durations.get(program_fingerprint, None)
         if duration_sec is None:
-            logging.warning("no duration stored. Using default")
+            logging.warning("no duration stored. Using default (key: " + program_fingerprint + " available values: " + ", ".join([key + ": " + str(self._durations.get(key)) for key in self._durations.keys()]) + ")")
             return 7222  # 2h
         else:
             return duration_sec
 
-    @property
-    def program_duration_hours(self) -> float:
-        return round(self.__program_duration_sec() / (60*60), 1)
-
     def __compute_remaining_secs_to_finish(self, start_date: str, duration_sec: int) -> int:
         remaining_secs_to_finish = int((datetime.fromisoformat(start_date) - datetime.now()).total_seconds()) + duration_sec
         if remaining_secs_to_finish < 0:
@@ -496,7 +503,6 @@ class FinishInAppliance(Appliance):
             try:
                 self._perform_put("/programs/active", json.dumps(data, indent=2), max_trials=3)
                 logging.info(self.name + " program " + self.program_selected + " starts at " + start_date + " (duration " + str(round(program_duration_sec/(60*60), 1)) + " h)")
-                self.__start_date = start_date
             except Exception as e:
                 logging.warning("error occurred by starting " + self.name + " with program " + self.program_selected + " at " + start_date + " (duration: " + str(round(program_duration_sec/(60*60), 1)) + " h) " + str(e))
         else:
